
MonMBM<- function(x, t=NULL, plotting=TRUE, title1="Original Sample",title2="Monotonized Sample", ylabel1="x(t)",ylabel2="xmon(t)") 
{
#Robust depth-based estimation of the structural mean in the time-warping model (Arribas-Gil and Romo, Biostatistics 2012)
#Ana Arribas-Gil, 2013  ana.arribas@uc3m.es


n <- nrow(x); p <- ncol(x) # n: number of observations (curves);  p: dimension of the data
x <- as.matrix(x) # Data matrix



if (n>1) {
   if (p == 1) {x <- t(x)}  
   
# Monotone transformation of the curves (as in Dupuy, Loubes and Maza, Statistics and Computing 2011)
	x.mon<- matrix(,n,p)
	for (i in 1:n) {
		x.mon[i,1]<-x[i,1]
		for (j in 2:p){
			x.mon[i,j]<-x.mon[i,j-1]+abs(x[i,j]-x[i,j-1])
		}
	}	
	
# Modified Band Median in the monotonized sample
# Fast Modified Band Depth computation for J=2 (as in Sun and Genton, Stat 2012) 
   rmat=apply(t(x.mon),1,rank,ties.method="max")  #in ties, both curves get higher rank
   down=rmat-1
   up=n-rmat
   mbd=(rowSums(up*down)/p+n-1)/(n*(n-1)/2)
   mbm.ind<-which.max(mbd)
   
# Curve in the original sample whose monotonized version is the deepest curve in the monotonized sample   
   mbm<-x[mbm.ind,]
   
# Graphics
  if (plotting) { 
     if (length(t)<1) {t<-1:p}  
     hues = seq(15, 375, length=n+1)
     color<-hcl(h=hues, l=65, c=100)[1:n]

     dev.new(width=12,height=6)
     layout(matrix(c(1,2), 1, 2, byrow = T))
     
     ### plot 1: original sample
     matplot(t,t(x),type="l",col =color,xlab="t",ylab=ylabel1,main=title1,lty=1)
     lines(t,mbm,col=1,lty=1,lwd=2)      
     ### plot 2: monotonized sample
     matplot(t,t(x.mon),type="l",col =color,xlab="t",ylab=ylabel2,main=title2,lty=1)
     lines(t,x.mon[mbm.ind,],col=1,lty=1,lwd=2)      
  } #END PLOTTING

}#END N>=1
return(list(deepest.curve.index=mbm.ind,deepest.curve=mbm,modified.band.depth=mbd,original.sample=x,monotonized.sample=x.mon) )

}
