#### Simulation Study 1 ####
############################
#Robust depth-based estimation of the structural mean in the time-warping model (Arribas-Gil and Romo, Biostatistics 2012)
#Ana Arribas-Gil, 2013  ana.arribas@uc3m.es

p<-200
n<-50

t<-seq(0,1,,p)
x.target<-sin(3*pi*t)+3*pi*t  #Target function
dev.new()
par(mfrow=c(2,2))
plot(t,x.target,type='l',lty=1,ylab="Target")

# Warping functions
mu<-1.5
a<-seq(-1,1,,n)

#Initial warping function
h<-exp(mu*a%*%t(t))-1  #matrix with n warping functions
 
q1<-(n-2)/2
q2<-(n+4)/2
q<-n/2
h[1:q1,]<-h[1:q1,]/(exp(mu*a[1:q1]%*%matrix(1,1,p))-1)
h[q2:n,]<-h[q2:n,]/(exp(mu*a[q2:n]%*%matrix(1,1,p))-1)
h[q,]<-t
h[q+1,]<-t
#matplot(t,t(h),type='l')

#Iterative procedure to generate warping functions
eps<-0.005
N<-2500
for (i in 1:N) {
	
	u<-runif(1)*(1-20*eps)+10*eps
     v<-runif(n)*2*eps+(u-eps)
     pu<-floor(u*(p-1))+1
     
     for (j in 1:n){
     	   w<-v[j]*t[1:pu]/u
        w<-append(w, ((1-v[j])/(1-u))*t[(pu+1):p]+(v[j]-u)/(1-u) )
                
        aux<-approx(w, h[j,], t, method="linear",rule=2)   ##composition of w and h in t
        h[j,]<-aux$y
     }
}
matplot(t,t(h),type='l',lty=1,col='gray',ylab="Warping functions",main="Original sample")

#Observed functions: composition of x.target with the warping functions
#Warping function inverses h^-1
h.inv<-matrix(0,n,p)
for (i in 1:n){
        aux<-approx(h[i,],t,t, method="linear",rule=2)   ##composition of w and h in t
        h.inv[i,]<-aux$y
}
matplot(t,t(h.inv),type='l',lty=1,col='gray',ylab="Inverse warping functions",main="Original sample")
#Observed functions
x<-matrix(0,n,p)
for (i in 1:n){
        aux<-approx(t,x.target,h.inv[i,], method="linear",rule=2)   ##composition of w and h in t
        x[i,]<-aux$y
}
matplot(t,t(x),type='l',lty=1,col='gray',ylab="Observed functions",main="Original sample")
#Target function and estimates
lines(t,x.target,lty=1)
lines(t,apply(x,2,mean),lty=1,col=2)

#Robust Depth based estimator
Depth.Est<-MonMBM(x,plotting=F)
lines(t,x[Depth.Est$deepest.curve.index,],lty=1,col=3)
leg.text<-c("Target", "Ptw Mean","Mon MBM")
legend("topleft",leg.text,col=c(1,2,3),lty=1)

#########################
## Contaminated sample ##
dev.new()
par(mfrow=c(2,2))
plot(t,x.target,type='l',lty=1,ylab="Target")
#We introduce 5 atypical warping functions
beta<-seq(10,14,,5)
h[1:5,]<-0.5+atan(beta%*%(2*t(t)-1))/(2*atan(beta))
matplot(t,t(h),type='l',lty=1,col='gray',ylab="Warping functions",main="Contaminated sample")
for (i in 1:5) {
    lines(t,h[i,],col=6)
}
#Warping function inverses h^-1
for (i in 1:5){
        aux<-approx(h[i,],t,t, method="linear",rule=2)   ##composition of w and h in t
        h.inv[i,]<-aux$y
}
matplot(t,t(h.inv),type='l',lty=1,col='gray',ylab="Inverse warping functions",main="Contaminated sample")
for (i in 1:5){
        lines(t,h.inv[i,],col=6)
}
#Observed functions
for (i in 1:5){
        aux<-approx(t,x.target,h.inv[i,], method="linear",rule=2)    ##composition of w and h in t
        x[i,]<-aux$y       
}
matplot(t,t(x),type='l',lty=1,col='gray',ylab="Observed functions",main="Contaminated sample")
for (i in 1:5){
        lines(t,x[i,],col=6)        
}
#Target function and estimates
lines(t,x.target,lty=1)
lines(t,apply(x,2,mean),lty=1,col=2)

#Robust Depth based estimator
Depth.Est<-MonMBM(x,plotting=F)
lines(t,x[Depth.Est$deepest.curve.index,],lty=1,col=3)
leg.text<-c("Target", "Ptw Mean","Mon MBM")
legend("topleft",leg.text,col=c(1,2,3),lty=1)
